package com.lucasmufato.bd2;

import com.lucasmufato.bd2.model.Employee;
import com.lucasmufato.bd2.repository.ConnectionManager;
import com.lucasmufato.bd2.repository.EmployeeDao;
import com.lucasmufato.bd2.repository.EmployeeDaoImplementation;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

public class AppTest {

  private static EmployeeDao controller;

  /**
   * Se ejecuta una sola ves. hace la conexion a la BD
   */
  @BeforeAll
  public static void beforeAll(){
    controller = new EmployeeDaoImplementation( ConnectionManager.getConection() );
  }

  /**
   * vacia la tabla para que los test encuentren la BD siempre en el mismo estado
   */
  @BeforeEach
  void setUp() {
    controller.removeAll();
  }

  /**
   * inserta un employee
   */
  @Test
  void insertEmployee() {
    Employee employee = new Employee(1,"lucas",35000);
    controller.insert(employee);
    Employee employee1 = controller.getById(1);
    Assertions.assertNotNull(employee1);
    Assertions.assertEquals(employee, employee1);
  }

  /**
   * inserta un employee y luego actualiza su salario
   */
  @Test
  void update() {
    Employee employee = new Employee(2,"micaela", 15000);
    controller.insert(employee);
    employee.setSalary(25000);
    controller.merge(employee);
    employee = null;
    Employee secondEmployee = controller.getById(2);
    Assertions.assertEquals(secondEmployee.getSalary(), 25000);
  }

  /**
   * borra un employee
   */
  @Test
  void removeOne() {
    Employee employee = new Employee(1, "lucas", 35000);
    controller.insert(employee);
    controller.remove(employee);
    Employee nullEmployee = controller.getById(1);
    Assertions.assertNull(nullEmployee);
  }

  /**
   * arma una lista de employees
   * @return employeeList
   */
  private List<Employee> armarListaEmployees(){
    List<Employee> employeeList = new ArrayList<>();
    employeeList.add( new Employee(1, "lucas", 35000) );
    employeeList.add( new Employee(2, "guillermo", 150000) );
    employeeList.add( new Employee(3, "lulu", 20000) );
    employeeList.add( new Employee(4, "romer", 45000) );
    employeeList.add( new Employee(5, "racker", 45000) );
    return employeeList;
  }

  /**
   * compara que los metodos del controlador findAllJPQL() y findAllCriteria() devuelven los mismos valores y
   * estos son los correctos
   */
  @Test
  void comparaObtenerAllEmployee() {
    List<Employee> employeeList = this.armarListaEmployees();
    employeeList.forEach( e -> controller.insert(e) );

    List<Employee> jpql = controller.findAll();

    Assertions.assertNotNull( jpql );
    Assertions.assertTrue( jpql.containsAll( employeeList ) );

  }

  /**
   * compara que los metodos del controlador findAllJPQLSalaryGreaterThan() y findAllCriteriaSalaryGreaterThan()
   * devuelven los mismos valores y estos son los correctos
   */
  @Test
  void comparaObtenerEmployeeConSalarioMayor() {
    final long salary = 30000;

    List<Employee> employeeList = this.armarListaEmployees();
    employeeList.forEach( e -> controller.insert(e) );

    List<Employee> query = controller.findAllSalaryGreaterThan(salary);
    List<Employee> highSalaryEmployees = employeeList.stream().
        filter(e -> e.getSalary() > salary)
        .collect( Collectors.toList() );

    Assertions.assertNotNull( query );
    Assertions.assertTrue( query.containsAll( highSalaryEmployees ) );

  }

  @Test
  void SQL_inyection_1() {
    List<Employee> employeeList = this.armarListaEmployees();
    employeeList.forEach( e -> controller.insert(e) );
    Employee employee = new Employee();
    employee.setId(10);
    employee.setSalary(1231);
    employee.setName("lucasmufato'); update employee set salary=1000000 where name='lucasmufato';--");
    controller.insert(employee);

    Employee empleadoModificado = controller.getById(10);
    Assertions.assertEquals( empleadoModificado.getSalary(), 1000000);
  }

}

